import logging

import botocore
from aiohttp.web_exceptions import HTTPNotFound

from dre_c_rule_publisher.utils.rmsutil import (
    read_config,
    get_dre_rule_config,
    get_env_config,
)

_logger = logging.getLogger(__name__)


def _download_s3_object(s3_client, s3_bucket, s3_key):
    try:
        return s3_client.get_object(Bucket=s3_bucket, Key=s3_key)
    except botocore.exceptions.ClientError as error:
        if error.response["Error"]["Code"] == "NoSuchKey":
            raise HTTPNotFound(reason=f"{s3_key} Not Found")
        raise Exception(
            f"Failed to Download Aggregated RuleSet due to Error {error.response['Error']['Code']}"
        ) from error


class DreCRuleEngine:
    def __init__(self, config, s3client, metrics):
        self.config = config
        self.metrics = metrics
        self.s3client = s3client

    async def get_dre_c_rules(self, env):
        with self.metrics.timer("rules.download.duration").time():
            s3_bucket = self.config.get("dre_c_rule", "s3_bucket")
            s3_rules_key = (
                self.config.get("dre_c_rule", "s3_dir")
                + "/"
                + env
                + "/"
                + self.config.get("dre_c_rule", "file")
            )

            s3_rules_config_key = (
                self.config.get("dre_c_rule", "s3_dir")
                + "/"
                + self.config.get("dre_c_rule", "config")
            )
            _logger.info(f"Fetching Dre Rules from {s3_bucket}/{s3_rules_key}")
            rules = _download_s3_object(self.s3client, s3_bucket, s3_rules_key)[
                "Body"
            ].read()
            config = (
                _download_s3_object(self.s3client, s3_bucket, s3_rules_config_key)[
                    "Body"
                ]
                .read()
                .decode("utf-8")
            )
            config_data = read_config(config)
            dre_c_rules_config = config_data["dre-c-rules"]
            env_config = get_env_config(
                dre_c_rules_config["environment"], self.config.get("env", "type")
            )
            env_name = env_config.get(self.config.get("env", "id"))

            if env_name is not None:
                env_name = self.config.get("env", "id")
            else:
                env_name = "default"

            return {
                "status": dre_c_rules_config["status"],
                "platform": dre_c_rules_config["platform"],
                "architecture": dre_c_rules_config["architecture"],
                "distribution": dre_c_rules_config["distribution"],
                "version_type": dre_c_rules_config["version_type"],
                "external_urn": dre_c_rules_config["external_urn"],
                "segment_scope_urn": dre_c_rules_config["segment_scope_urn"],
                "segment_name": dre_c_rules_config["segment_name"],
                "segment_type": dre_c_rules_config["segment_type"],
                "segment_description": dre_c_rules_config["segment_description"],
                "major_version": get_dre_rule_config(
                    env_config,
                    env_name,
                    "major_version",
                ),
                "minor_version": get_dre_rule_config(
                    env_config,
                    env_name,
                    "minor_version",
                ),
                "patch": get_dre_rule_config(
                    env_config,
                    env_name,
                    "patch",
                ),
                "build": get_dre_rule_config(
                    env_config,
                    env_name,
                    "build",
                ),
                "segment_data": rules,
            }

    async def shutdown(self, _app=None):
        pass
