import json
import os

import requests

from dre_c_rule_publisher.model.models import Segment, SegmentData, Assignment
from dre_c_rule_publisher.utils.rmsutil import check_response_status


def get_auth_token():
    auth_token = (
        os.getenv("API_KEY", "RMS_SERVICE_CONNECTOR_SECRET")
        + "/"
        + os.getenv("CONNECTOR_ID", "RMS_SERVICE_CONNECTOR_ID")
    )
    return auth_token


class RMSRulesManager:
    def __init__(self, dre_c_rules_config, config):
        self.host = config.get("rms", "host_name")
        self.session = requests.session()
        self.session.verify = False
        self.session.headers = {
            "Content-Type": "application/json",
            "X-Auth-Token": get_auth_token(),
        }

        self.status = dre_c_rules_config["status"]
        self.platform = dre_c_rules_config["platform"]
        self.architecture = dre_c_rules_config["architecture"]
        self.distribution = dre_c_rules_config["distribution"]
        self.version_type = dre_c_rules_config["version_type"]
        self.external_urn = dre_c_rules_config["external_urn"]
        self.segment_scope_urn = dre_c_rules_config["segment_scope_urn"]
        self.segment_name = dre_c_rules_config["segment_name"]
        self.segment_type = dre_c_rules_config["segment_type"]
        self.segment_description = dre_c_rules_config["segment_description"]

        self.major_version = dre_c_rules_config["major_version"]
        self.minor_version = dre_c_rules_config["minor_version"]
        self.patch = dre_c_rules_config["patch"]
        self.build = dre_c_rules_config["build"]

    def search_segment(self):
        """
        Function to Search Segment for given External URN
        :return: segment
        """
        search_segment_url = f"https://{self.host}/rules/private/v3/segments/_search"
        search_segment_request = {"criteria": {"external_urn": [self.external_urn]}}
        response = self.session.post(search_segment_url, json=search_segment_request)
        check_response_status(response)
        return response.json()

    def create_segment(self):
        """
        Function to Create New Segment
        :return: segment
        """
        create_segment_url = (
            f"https://{self.host}/rules/private/v1/segments/{self.segment_scope_urn}"
        )

        create_segment_request = Segment(
            self.segment_scope_urn,
            self.status,
            self.segment_name,
            self.segment_type,
            self.segment_description,
            self.external_urn,
        )

        response = self.session.post(create_segment_url, json=create_segment_request)

        check_response_status(response)
        return response.json()

    def find_segment_data_by_version(self, segment_id):
        """
        Function to find Segment Data by Version
        :param segment_id:
        :return: segment data
        """

        v_list = [
            self.major_version,
            self.minor_version,
            self.patch,
            self.build,
        ]

        rule_version = ".".join(str(v) for v in v_list)

        path_param_list = [
            self.segment_scope_urn,
            str(segment_id),
            "data",
            self.platform,
            self.architecture,
            self.distribution,
            rule_version,
        ]

        path_params = "/".join(str(v) for v in path_param_list)

        find_segment_by_version_url = (
            f"https://{self.host}/rules/private/v2/segments/{path_params}"
        )

        response = self.session.get(find_segment_by_version_url)
        check_response_status(response)

        return response

    def create_segment_data(self, segment_id, rules):
        """
        Function to Create New Segment Data
        :param segment_id:
        :param rules:
        :return: segment data
        """

        create_segment_data_url = f"https://{self.host}/rules/private/v2/segments/{self.segment_scope_urn}/{segment_id}/data"

        create_segment_data_request = SegmentData(
            segment_id,
            self.major_version,
            self.minor_version,
            self.patch,
            self.build,
            json.dumps(rules),
            self.platform,
            self.architecture,
            self.distribution,
        )

        response = self.session.post(
            create_segment_data_url, json=create_segment_data_request
        )

        check_response_status(response)

        return response

    def find_assignment(self, segment_id):
        """
        Checks if segment has any current assignments
        :param segment_id:
        :return: assignment
        """

        path_param_list = [
            self.segment_scope_urn,
            str(segment_id),
            "assignments",
            self.platform,
            self.architecture,
            self.distribution,
        ]

        path_params = "/".join(str(v) for v in path_param_list)

        find_assignment_url = (
            f"https://{self.host}/rules/private/v2/segments/{path_params}"
        )

        response = self.session.get(find_assignment_url)

        check_response_status(response)

        return response

    def create_assignment(self, segment_id):
        """
        Function to create New Assignment for given Segment Id
        :param segment_id:
        :return: assignment
        """
        create_assignment_url = f"https://{self.host}/rules/private/v2/segments/{self.segment_scope_urn}/{segment_id}/assignments"
        create_assignment_request = Assignment(
            self.segment_scope_urn,
            segment_id,
            self.status,
            self.platform,
            self.architecture,
            self.distribution,
            self.version_type,
            self.major_version,
            self.segment_type,
        )
        response = self.session.post(
            create_assignment_url, json=create_assignment_request
        )

        check_response_status(response)

        return response

    def update_assignment(self, segment_id, assignment_id):
        """
        Function to update assignment (rule set) for given segment id and assignment id
        :param segment_id:
        :param assignment_id:
        :return: assignment
        """
        update_assignment_url = f"https://{self.host}/rules/private/v2/segments/{self.segment_scope_urn}/{segment_id}/assignments"

        update_assignment_request = Assignment(
            self.segment_scope_urn,
            segment_id,
            self.status,
            self.platform,
            self.architecture,
            self.distribution,
            self.version_type,
            self.major_version,
            self.segment_type,
            assignment_id,
        )

        response = self.session.put(
            update_assignment_url, json=update_assignment_request
        )

        check_response_status(response)

        return response
