import json
import logging
import os
import sys
import time
import urllib3
from pscr_common.log import clear_logging_handlers, setup_logging
from pscr_common.metrics import get_reporter
from pscr_common.metadata import MetadataProvider
from pyformance import MetricsRegistry
from rich.tree import Tree
from rich import print

from dre_c_rule_publisher.publisher.dre_c_rule_client import DreCRuleClient
from dre_c_rule_publisher.utils.config import Config
from dre_c_rule_publisher.publisher.rms_rules_manager import RMSRulesManager
from dre_c_rule_publisher.utils.rmsutil import (
    get_depl_manifest,
    get_dre_rule_config,
    get_env_config,
)

urllib3.disable_warnings()
_logger = logging.getLogger(__name__)


def str_to_bool(value):
    if value.lower() == "true":
        return True
    if value.lower() == "false":
        return False
    raise Exception("Invalid boolean string passed as env var")


def main():
    config = Config.create()

    clear_logging_handlers()
    setup_logging(
        level=config.get("log", "level", fallback="INFO"),
        svc_version=config.get("service", "version"),
        env_id=config.get("env", "id"),
        svc_id="TODO",
    )

    _logger.info("Dre-C Rule Publisher Started")

    metrics_registry = MetricsRegistry()
    env = os.environ.get("ENV_ID", "unknown")
    env_meta = MetadataProvider().get_metadata()
    metrics_reporter = get_reporter(
        metrics_registry,
        env,
        env_meta.task_id,
        "dre_c_rule_publisher",
        env_meta.image.split(":")[-1],
        os.getpid(),
    )

    if str_to_bool(config.get("dre_c_rule", "dry_run")):
        # Making thread sleep to print logs in order
        time.sleep(0.1)
        _logger.info("Dre-C Rule Publisher running in DRY RUN Mode")
        # Making thread sleep to print logs in order
        time.sleep(0.1)
        _logger.info("Dre-C Rule Publisher Finished")
        if metrics_reporter is not None:
            metrics_reporter.report_now()
        sys.exit()

    if str_to_bool(config.get("dre_c_rule", "enable_api_mode")):
        dre_c_rule_client = DreCRuleClient(config)
        resp = dre_c_rule_client.get_dre_rules()
        if resp.status_code == 404:
            _logger.info("No Rules Found")
            metrics_registry.counter("dre_c_rule_publisher.error.no_rules_found")
            if metrics_reporter is not None:
                metrics_reporter.report_now()
                sys.exit()

        dre_c_rules_config = resp.json()

    else:

        dre_c_rules_config = get_depl_manifest("dre-c-rules")

        env_config = get_env_config(
            dre_c_rules_config["environment"], config.get("env", "type")
        )
        env_name = env_config.get(config.get("env", "id"))

        if env_name is not None:
            env_name = config.get("env", "id")
        else:
            env_name = "default"
            _logger.warning(
                f"No Configuration Found for env {config.get('env', 'id')} using default rule config"
            )

        dre_c_rules_config["major_version"] = get_dre_rule_config(
            env_config,
            env_name,
            "major_version",
        )
        dre_c_rules_config["minor_version"] = get_dre_rule_config(
            env_config,
            env_name,
            "minor_version",
        )
        dre_c_rules_config["patch"] = get_dre_rule_config(
            env_config,
            env_name,
            "patch",
        )

        dre_c_rules_config["build"] = get_dre_rule_config(
            env_config,
            env_name,
            "build",
        )

        rules = os.path.join(
            os.path.dirname(__file__),
            f"../dre_c_rules/{config.get('env', 'type')}/{env_name}",
            "rules.json",
        )

        if not os.path.exists(rules):
            rules = os.path.join(
                os.path.dirname(__file__),
                f"../dre_c_rules/{config.get('env', 'type')}/default",
                "rules.json",
            )

        with open(rules) as f:
            drec_rules = json.load(f)
            dre_c_rules_config["segment_data"] = drec_rules

    rms_session = RMSRulesManager(dre_c_rules_config, config)

    with metrics_registry.timer("dre_c_rule_publisher.publish.duration").time():
        # Step 2: Get Segment Id
        segment_id = find_segment(rms_session)
        # Step 3: Search for Segment Data by Version if not found create segment data with new rule set
        segment_data_id = find_segment_data(rms_session, segment_id, dre_c_rules_config)
        # Step 4: Update Assignment  - if there are no previous assignments  create new assignment
        assign_resp = update_assignment(rms_session, segment_id, config)

    if metrics_reporter is not None:
        metrics_reporter.report_now()

    log_updates(dre_c_rules_config, segment_id, segment_data_id, assign_resp)

    _logger.info("Dre-C Rule Publisher Finished")


def find_segment(rms_session):
    """
    Check if Segment Exist for given external_urn, if not found create new segment.
    :param rms_session:
    :return: segment_id
    """
    search_segment_response = rms_session.search_segment()

    if len(search_segment_response["results"]) == 0:
        _logger.info(
            f"Segment not found for external_url {rms_session.external_urn}.Creating New Segment!!"
        )
        search_segment_response = rms_session.create_segment()
        segment_id = search_segment_response["segment_id"]
    else:
        _logger.info(
            f"Found Segment for the External URN: {search_segment_response['results'][0]['external_urn']}"
        )
        segment_id = search_segment_response["results"][0]["segment_id"]

    _logger.info(f"Segment Id: {segment_id}")
    return segment_id


def find_segment_data(rms_session, segment_id, dre_c_config):
    """
    Check if Segment Data Exist for given Segment Id and Version, if not found create new Segment Data(rule set)
    :param dre_c_config:
    :param rms_session:
    :param segment_id:
    :return:
    """
    segment_data_response = rms_session.find_segment_data_by_version(segment_id)
    if segment_data_response.status_code == 404:
        _logger.info("Segment Data not found. Creating New Segment Data.")
        segment_data_response = rms_session.create_segment_data(
            segment_id, dre_c_config["segment_data"]
        )
    else:
        v_list = [
            dre_c_config["major_version"],
            dre_c_config["minor_version"],
            dre_c_config["patch"],
            dre_c_config["build"],
        ]
        rule_version = ".".join(str(v) for v in v_list)
        _logger.info(
            f"Found Segment Data for Segment: {segment_id} and Version: {rule_version}"
        )

    segment_data_id = segment_data_response.json()["segment_data_id"]

    _logger.info(f"Segment Data Id:  {segment_data_id}")

    return segment_data_id


def update_assignment(rms_session, segment_id, config):
    """
    Check if there are any previous assignments for given Segment , if found update else create new assignment
    :param config:
    :param rms_session:
    :param segment_id:
    :return:
    """
    curr_assignment_response = rms_session.find_assignment(segment_id)

    if curr_assignment_response.status_code == 404:
        _logger.info(
            f"Assignment not found for Segment {segment_id}.Creating New Assignment."
        )
        update_assignment_response = rms_session.create_assignment(segment_id)
        if update_assignment_response.status_code != 200:
            _logger.critical(
                f"Rule Assignment Failed due to {update_assignment_response.json()['message']}"
            )
            raise Exception
    else:
        curr_major_version = curr_assignment_response.json()["version_major"]
        if curr_major_version == rms_session.major_version:
            _logger.info(
                f"Current RMS Assignment Major Version {curr_major_version} for Segment: {segment_id} in {config.get('env', 'type')}-{config.get('env', 'id')} is same as Deployment Manifest Major Version!"
                f" Not Updating  RMS!"
            )
            _logger.info("Dre-C Rule Publisher Finished")
            sys.exit()
        assignment_id = curr_assignment_response.json()["assignment_id"]
        _logger.info(
            f"Current RMS Assignment({assignment_id}) for Segment: {segment_id} with Major Version is "
            f"{curr_major_version}. Updating Assignment"
        )
        update_assignment_response = rms_session.update_assignment(
            segment_id, assignment_id
        )
        if update_assignment_response.status_code != 200:
            _logger.critical(
                f"Rule Assignment Update Failed due to {update_assignment_response.json()['message']}"
            )

    assignment_list = {
        "curr_assignment": curr_assignment_response,
        "update_assignment": update_assignment_response,
    }

    return assignment_list


def log_updates(dre_c_rules_config, segment_id, segment_data_id, assign_resp):
    tree = Tree("RULES ASSIGNMENT VIEW!!!")
    tree.add(f"SEGMENT SCOPE URN : {dre_c_rules_config['segment_scope_urn']}")
    tree.add(f"SEGMENT TYPE : {dre_c_rules_config['segment_type']}")
    tree.add(f"EXTERNAL URN  : {dre_c_rules_config['external_urn']}")
    segment = tree.add(f"SEGMENT : {segment_id}")
    segment.add(f"SEGMENT DATA : {segment_data_id}")
    prev_major_version = None
    updated_major_version = None

    for k, v in assign_resp.items():
        if k == "curr_assignment" and v.status_code != 404:
            assignment_id = v.json()["assignment_id"]
            prev_major_version = v.json()["version_major"]
        elif k == "update_assignment" and v is not None:
            assignment_id = v.json()["assignment_id"]
            updated_major_version = v.json()["version_major"]

    assignment = segment.add(f"ASSIGNMENT ID : {assignment_id}")
    assignment.add(f"PREVIOUS MAJOR VERSION : {prev_major_version}")
    assignment.add(f"UPDATED MAJOR VERSION : {updated_major_version}")

    _logger.info(print(tree))


if __name__ == "__main__":
    main()
