import asyncio
import logging
import time

_logger = logging.getLogger(__name__)


class BackgroundTask:
    """
    BackgroundTask is an async task linked to the lifecycle of `web.Application`. The task will be scheduled when `register` is called
    and will be suspended when the underlying `web.Application` is terminated, e.g. after SIGTERM.

    BackgroundTask can work in one of 2 modes depending on the `execute_once` parameter.
    If `execute_once` is false, the background task will be scheduled every `loop_sec` seconds.
    If `execute_once` is true, the background task will be scheduled exactly once.

    In both cases, exceptions are suppressed and logged, i.e. exceptions will not cause BackgroundTask exiting,
    and it will continue scheduling next executions. For `execute_once == true`, this means the task will be scheduled
    every `loop_sec` seconds until a successful execution.

    The `sleep_first` parameter controls whether the first execution will be scheduled immediately after `register` is called
    or whether it will be scheduled `loop_sec` seconds after `register` is called.
    """

    def __init__(self, name, loop_sec=60, sleep_first=True, execute_once=False):
        self.name = name
        self.loop_sec = loop_sec
        self.sleep_first = sleep_first
        self.execute_once = execute_once

    async def register(self, app, instance_number=None) -> None:
        if instance_number is not None:
            self.name = f"{self.name}-{instance_number}"

        app.cleanup_ctx.append(self.__context)

    async def __context(self, app) -> None:
        background_task = asyncio.get_event_loop().create_task(self.__run(app))
        yield

        background_task.cancel()
        await background_task

    async def __run(self, app) -> None:
        start = time.time()
        while True:
            try:
                await asyncio.sleep(
                    (self.loop_sec - ((time.time() - start) % self.loop_sec))
                    * self.sleep_first
                )
                await self.execute(app)

                self.sleep_first = True
                if self.execute_once:
                    return
            except asyncio.CancelledError:
                _logger.info("Task '%s' cancelled", self.name)
                return
            except:
                _logger.exception(f"Unknown error during {self.name}")

    async def execute(self, app) -> None:
        raise NotImplementedError
