import logging
import asyncio
import os

from aiohttp import web
from aio_pyformance.registry import MetricsRegistry

from pscr_common.log import setup_logging, clear_logging_handlers, PSCRAccessLogger
from pscr_common.utils import MB
from pscr_common.web.auth import auth_middleware_factory, AuthClient
from pscr_common.aws import get_boto3_client
from pscr_common.metrics import get_reporter
from pscr_common.metadata import MetadataProvider

from dre_c_rule_publisher.engine.dre_c_rule_engine import DreCRuleEngine
from dre_c_rule_publisher.utils.config import Config
from dre_c_rule_publisher.utils.metric_reporter_task import MetricReporterTask
from dre_c_rule_publisher.web import internal_drecrules_routes, error_handler

_logger = logging.getLogger(__name__)


def build_internal_drecrules_app(dre_engine):
    app = web.Application()

    table = internal_drecrules_routes.build_route_table(dre_engine)
    table.setup_routes_and_auth(app)

    return app


async def create_metric_reporting_task(app, metrics_registry):
    env = os.environ.get("ENV_ID", "unknown")
    env_meta = MetadataProvider().get_metadata()
    metrics_reporter = get_reporter(
        metrics_registry,
        env,
        env_meta.task_id,
        "dre_c_rule_publisher",
        env_meta.image.split(":")[-1],
        os.getpid(),
    )
    if metrics_reporter:
        reporter_task = MetricReporterTask(metrics_reporter)
        await reporter_task.register(app)


async def build_app() -> web.Application:
    config = Config.create()

    clear_logging_handlers()
    setup_logging(
        level=config.get("log", "level"),
        svc_version=os.environ.get("CONTAINERVERSION", "UNKNOWN"),
        env_id=os.environ.get("ENV_ID", "UNKNOWN"),
        svc_id="TODO",
    )

    metrics_registry = MetricsRegistry()

    s3client = get_boto3_client("s3")

    dre_engine = DreCRuleEngine(config, s3client, metrics_registry)

    # Convert max request size megabytes to bytes
    max_request_size_bytes = int(os.environ.get("MAX_REQUEST_SIZE_MB", 10)) * MB

    # RBAC client
    auth_client = AuthClient()

    app = web.Application(
        middlewares=[
            auth_middleware_factory(auth_client),
            error_handler.error_middleware_factory(metrics_registry),
        ],
        client_max_size=max_request_size_bytes,
        handler_args={
            "access_log_class": PSCRAccessLogger,
            "access_log": logging.getLogger("access_log"),
        },
    )

    await auth_client.initialize(app)

    internal_drecrules_app = build_internal_drecrules_app(dre_engine)
    app.add_subapp("/internal/drecrules/", internal_drecrules_app)

    await create_metric_reporting_task(app=app, metrics_registry=metrics_registry)

    app.on_shutdown.append(dre_engine.shutdown)
    return app


if __name__ == "__main__":
    webapp = asyncio.get_event_loop().run_until_complete(build_app())

    web.run_app(webapp, port=5006)
