import logging

import botocore
from aiohttp import web
from aiohttp.web_exceptions import HTTPNotFound
from pscr_common.web.auth import AuthRouteTableDef, Action

from dre_c_rule_publisher.engine.dre_c_rule_engine import DreCRuleEngine
from dre_c_rule_publisher.utils import json_dumps
from dre_c_rule_publisher.web.auth import Resource

_logger = logging.getLogger(__name__)


def make_response_json(data=None) -> web.Response:
    if data is None:
        return web.Response(status=204)

    return web.json_response(data, dumps=json_dumps)


def build_route_table(engine: DreCRuleEngine) -> AuthRouteTableDef:
    routes = AuthRouteTableDef()

    @routes.get("/healthcheck")
    async def healthcheck(request: web.Request):
        """
        ---
        summary: Healthcheck
        description: Successful response indicates service reachability.
        produces:
        - application/octet-stream
        responses:
            204:
                description: service is available
        """
        request.do_not_log = True

        return make_response_json()

    @routes.get("/v1/environment/{env}", Resource.internal, [Action.execute])
    async def get_dre_c_rules(request: web.Request):
        """
        :param request:
        :return:
        """
        try:
            resp = await engine.get_dre_c_rules(request.match_info["env"])
        except botocore.exceptions.ClientError as error:
            if error.response["Error"]["Code"] == "NoSuchKey":
                raise HTTPNotFound(reason="No Rules Found") from error
            raise Exception(
                f"Failed to Download Aggregated RuleSet due to Error {error.response['Error']['Code']}"
            ) from error

        return make_response_json(resp)

    return routes
